@@ -52,6 +52,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #ifdef HAVE_EXIF
+#include "eom-exif-util.h"
 #include <libexif/exif-data.h>
 #include <libexif/exif-utils.h>
 #include <libexif/exif-loader.h>
@@ -79,7 +80,7 @@ enum {
 	SIGNAL_LAST
 };
 
-static gint signals[SIGNAL_LAST];
+static guint signals[SIGNAL_LAST] = { 0 };
 
 static GList *supported_mime_types = NULL;
 
@@ -148,6 +149,7 @@ eom_image_free_mem_private (EomImage *image)
 #endif
 
 		priv->status = EOM_IMAGE_STATUS_UNKNOWN;
+		priv->metadata_status = EOM_IMAGE_METADATA_NOT_READ;
 	}
 }
 
@@ -180,10 +182,7 @@ eom_image_dispose (GObject *object)
 		priv->file_type = NULL;
 	}
 
-	if (priv->status_mutex) {
-		g_mutex_free (priv->status_mutex);
-		priv->status_mutex = NULL;
-	}
+	g_mutex_clear (&priv->status_mutex);
 
 	if (priv->trans) {
 		g_object_unref (priv->trans);
@@ -292,7 +291,8 @@ eom_image_init (EomImage *img)
 	img->priv->width = -1;
 	img->priv->height = -1;
 	img->priv->modified = FALSE;
-	img->priv->status_mutex = g_mutex_new ();
+	img->priv->file_is_changed = FALSE;
+	g_mutex_init (&img->priv->status_mutex);
 	img->priv->status = EOM_IMAGE_STATUS_UNKNOWN;
         img->priv->metadata_status = EOM_IMAGE_METADATA_NOT_READ;
 	img->priv->is_monitored = FALSE;
@@ -478,9 +478,9 @@ do_emit_size_prepared_signal (EomImage *img)
 static void
 eom_image_emit_size_prepared (EomImage *img)
 {
-	gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE,
-				   (GSourceFunc) do_emit_size_prepared_signal,
-				   g_object_ref (img), g_object_unref);
+	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+	                (GSourceFunc) do_emit_size_prepared_signal,
+	                 g_object_ref (img), g_object_unref);
 }
 
 static gboolean
@@ -531,12 +531,12 @@ eom_image_size_prepared (GdkPixbufLoader *loader,
 
 	img = EOM_IMAGE (data);
 
-	g_mutex_lock (img->priv->status_mutex);
+	g_mutex_lock (&img->priv->status_mutex);
 
 	img->priv->width = width;
 	img->priv->height = height;
 
-	g_mutex_unlock (img->priv->status_mutex);
+	g_mutex_unlock (&img->priv->status_mutex);
 
 #ifdef HAVE_EXIF
 	if (img->priv->threadsafe_format && (!img->priv->autorotate || img->priv->exif))
@@ -828,12 +828,12 @@ eom_image_set_exif_data (EomImage *img, EomMetadataReader *md_reader)
 	priv = img->priv;
 
 #ifdef HAVE_EXIF
-	g_mutex_lock (priv->status_mutex);
+	g_mutex_lock (&priv->status_mutex);
 	if (priv->exif) {
 		exif_data_unref (priv->exif);
 	}
 	priv->exif = eom_metadata_reader_get_exif_data (md_reader);
-	g_mutex_unlock (priv->status_mutex);
+	g_mutex_unlock (&priv->status_mutex);
 
 	priv->exif_chunk = NULL;
 	priv->exif_chunk_len = 0;
@@ -1151,6 +1151,8 @@ eom_image_real_load (EomImage *img,
 				priv->file_type = gdk_pixbuf_format_get_name (format);
 			}
 
+			priv->file_is_changed = FALSE;
+
 			/* If it's non-threadsafe loader, then trigger window
  			 * showing in the end of the process. */
 			if (!priv->threadsafe_format)
@@ -1194,17 +1196,17 @@ eom_image_has_data (EomImage *img, EomImageData req_data)
 	priv = img->priv;
 
 	if ((req_data & EOM_IMAGE_DATA_IMAGE) > 0) {
-		req_data = (req_data & !EOM_IMAGE_DATA_IMAGE);
+		req_data = (req_data & ~EOM_IMAGE_DATA_IMAGE);
 		has_data = has_data && (priv->image != NULL);
 	}
 
 	if ((req_data & EOM_IMAGE_DATA_DIMENSION) > 0 ) {
-		req_data = (req_data & !EOM_IMAGE_DATA_DIMENSION);
+		req_data = (req_data & ~EOM_IMAGE_DATA_DIMENSION);
 		has_data = has_data && (priv->width >= 0) && (priv->height >= 0);
 	}
 
 	if ((req_data & EOM_IMAGE_DATA_EXIF) > 0) {
-		req_data = (req_data & !EOM_IMAGE_DATA_EXIF);
+		req_data = (req_data & ~EOM_IMAGE_DATA_EXIF);
 #ifdef HAVE_EXIF
 		has_data = has_data && (priv->exif != NULL);
 #else
@@ -1213,7 +1215,7 @@ eom_image_has_data (EomImage *img, EomImageData req_data)
 	}
 
 	if ((req_data & EOM_IMAGE_DATA_XMP) > 0) {
-		req_data = (req_data & !EOM_IMAGE_DATA_XMP);
+		req_data = (req_data & ~EOM_IMAGE_DATA_XMP);
 #ifdef HAVE_EXEMPI
 		has_data = has_data && (priv->xmp != NULL);
 #endif
@@ -1253,9 +1255,11 @@ eom_image_load (EomImage *img, EomImageData data2read, EomJob *job, GError **err
 
 #ifdef HAVE_EXIF
 	/* Check that the metadata was loaded at least once before
-	 * trying to autorotate. */
+	 * trying to autorotate. Also only an imatge load job should try to
+	 * autorotate and image */
 	if (priv->autorotate && 
-	    priv->metadata_status == EOM_IMAGE_METADATA_READY) {
+	    priv->metadata_status == EOM_IMAGE_METADATA_READY &&
+	      data2read & EOM_IMAGE_DATA_IMAGE) {
 		eom_image_real_autorotate (img);
 	}
 #endif
@@ -1303,6 +1307,14 @@ eom_image_set_thumbnail (EomImage *img, GdkPixbuf *thumbnail)
 	}
 }
 
+/**
+ * eom_image_get_pixbuf:
+ * @img: a #EomImage
+ *
+ * Gets the #GdkPixbuf of the image
+ *
+ * Returns: (transfer full): a #GdkPixbuf
+ **/
 GdkPixbuf *
 eom_image_get_pixbuf (EomImage *img)
 {
@@ -1310,9 +1322,9 @@ eom_image_get_pixbuf (EomImage *img)
 
 	g_return_val_if_fail (EOM_IS_IMAGE (img), NULL);
 
-	g_mutex_lock (img->priv->status_mutex);
+	g_mutex_lock (&img->priv->status_mutex);
 	image = img->priv->image;
-	g_mutex_unlock (img->priv->status_mutex);
+	g_mutex_unlock (&img->priv->status_mutex);
 
 	if (image != NULL) {
 		g_object_ref (image);
@@ -1331,15 +1343,21 @@ eom_image_get_profile (EomImage *img)
 }
 #endif
 
+/**
+ * eom_image_get_thumbnail:
+ * @img: a #EomImage
+ *
+ * Gets the thumbnail pixbuf for @img
+ *
+ * Returns: (transfer full): a #GdkPixbuf with a thumbnail
+ **/
 GdkPixbuf *
 eom_image_get_thumbnail (EomImage *img)
 {
 	g_return_val_if_fail (EOM_IS_IMAGE (img), NULL);
 
 	if (img->priv->thumbnail != NULL) {
-		g_object_ref (img->priv->thumbnail);
-
-		return img->priv->thumbnail;
+		return g_object_ref (img->priv->thumbnail);
 	}
 
 	return NULL;
@@ -1431,6 +1449,110 @@ transfer_progress_cb (goffset cur_bytes,
 	}
 }
 
+static void
+tmp_file_restore_unix_attributes (GFile *temp_file,
+				  GFile *target_file)
+{
+	GFileInfo *file_info;
+	guint      uid;
+	guint      gid;
+	guint      mode;
+	guint      mode_mask = 00600;
+
+	GError    *error = NULL;
+
+	g_return_if_fail (G_IS_FILE (temp_file));
+	g_return_if_fail (G_IS_FILE (target_file));
+
+	/* check if file exists */
+	if (!g_file_query_exists (target_file, NULL)) {
+		eom_debug_message (DEBUG_IMAGE_SAVE,
+				   "Target file doesn't exist. Setting default attributes.");
+		return;
+	}
+
+	/* retrieve UID, GID, and MODE of the original file info */
+	file_info = g_file_query_info (target_file,
+				       "unix::uid,unix::gid,unix::mode",
+				       G_FILE_QUERY_INFO_NONE,
+				       NULL,
+				       &error);
+
+	/* check that there aren't any error */
+	if (error != NULL) {
+		eom_debug_message (DEBUG_IMAGE_SAVE,
+				   "File information not available. Setting default attributes.");
+
+		/* free objects */
+		g_object_unref (file_info);
+		g_clear_error (&error);
+
+		return;
+	}
+
+	/* save UID, GID and MODE values */
+	uid = g_file_info_get_attribute_uint32 (file_info,
+						G_FILE_ATTRIBUTE_UNIX_UID);
+
+	gid = g_file_info_get_attribute_uint32 (file_info,
+						G_FILE_ATTRIBUTE_UNIX_GID);
+
+	mode = g_file_info_get_attribute_uint32 (file_info,
+						 G_FILE_ATTRIBUTE_UNIX_MODE);
+
+	/* apply default mode mask to file mode */
+	mode |= mode_mask;
+
+	/* restore original UID, GID, and MODE into the temporal file */
+	g_file_set_attribute_uint32 (temp_file,
+				     G_FILE_ATTRIBUTE_UNIX_UID,
+				     uid,
+				     G_FILE_QUERY_INFO_NONE,
+				     NULL,
+				     &error);
+
+	/* check that there aren't any error */
+	if (error != NULL) {
+		eom_debug_message (DEBUG_IMAGE_SAVE,
+				   "You do not have the permissions necessary to change the file UID.");
+
+		g_clear_error (&error);
+	}
+
+	g_file_set_attribute_uint32 (temp_file,
+				     G_FILE_ATTRIBUTE_UNIX_GID,
+				     gid,
+				     G_FILE_QUERY_INFO_NONE,
+				     NULL,
+				     &error);
+
+	/* check that there aren't any error */
+	if (error != NULL) {
+		eom_debug_message (DEBUG_IMAGE_SAVE,
+				   "You do not have the permissions necessary to change the file GID. Setting user default GID.");
+
+		g_clear_error (&error);
+	}
+
+	g_file_set_attribute_uint32 (temp_file,
+				     G_FILE_ATTRIBUTE_UNIX_MODE,
+				     mode,
+				     G_FILE_QUERY_INFO_NONE,
+				     NULL,
+				     &error);
+
+	/* check that there aren't any error */
+	if (error != NULL) {
+		eom_debug_message (DEBUG_IMAGE_SAVE,
+				   "You do not have the permissions necessary to change the file MODE.");
+
+		g_clear_error (&error);
+	}
+
+	/* free objects */
+	g_object_unref (file_info);
+}
+
 static gboolean
 tmp_file_move_to_uri (EomImage *image,
 		      GFile *tmpfile,
@@ -1441,6 +1563,10 @@ tmp_file_move_to_uri (EomImage *image,
 	gboolean result;
 	GError *ioerror = NULL;
 
+	/* try to restore target file unix attributes */
+	tmp_file_restore_unix_attributes (tmpfile, file);
+
+	/* replace target file with temporal file */
 	result = g_file_move (tmpfile,
 			      file,
 			      (overwrite ? G_FILE_COPY_OVERWRITE : 0) |
@@ -1873,38 +1999,46 @@ eom_image_cancel_load (EomImage *img)
 
 	priv = img->priv;
 
-	g_mutex_lock (priv->status_mutex);
+	g_mutex_lock (&priv->status_mutex);
 
 	if (priv->status == EOM_IMAGE_STATUS_LOADING) {
 		priv->cancel_loading = TRUE;
 	}
 
-	g_mutex_unlock (priv->status_mutex);
+	g_mutex_unlock (&priv->status_mutex);
 }
 
-gpointer
+#ifdef HAVE_EXIF
+ExifData *
 eom_image_get_exif_info (EomImage *img)
 {
 	EomImagePrivate *priv;
-	gpointer data = NULL;
+	ExifData *data = NULL;
 
 	g_return_val_if_fail (EOM_IS_IMAGE (img), NULL);
 
 	priv = img->priv;
 
-#ifdef HAVE_EXIF
-	g_mutex_lock (priv->status_mutex);
+	g_mutex_lock (&priv->status_mutex);
 
 	exif_data_ref (priv->exif);
 	data = priv->exif;
 
-	g_mutex_unlock (priv->status_mutex);
-#endif
+	g_mutex_unlock (&priv->status_mutex);
 
 	return data;
 }
+#endif
 
-
+/**
+ * eom_image_get_xmp_info:
+ * @img: a #EomImage
+ *
+ * Gets the XMP info for @img or NULL if compiled without
+ * libexempi support.
+ *
+ * Returns: (transfer full): the xmp data
+ **/
 gpointer
 eom_image_get_xmp_info (EomImage *img)
 {
@@ -1916,15 +2050,23 @@ eom_image_get_xmp_info (EomImage *img)
  	priv = img->priv;
 
 #ifdef HAVE_EXEMPI
- 	g_mutex_lock (priv->status_mutex);
+	g_mutex_lock (&priv->status_mutex);
  	data = (gpointer) xmp_copy (priv->xmp);
- 	g_mutex_unlock (priv->status_mutex);
+	g_mutex_unlock (&priv->status_mutex);
 #endif
 
  	return data;
 }
 
 
+/**
+ * eom_image_get_file:
+ * @img: a #EomImage
+ *
+ * Gets the #GFile associated with @img
+ *
+ * Returns: (transfer full): a #GFile
+ **/
 GFile *
 eom_image_get_file (EomImage *img)
 {
@@ -2046,6 +2188,13 @@ compare_quarks (gconstpointer a, gconstpointer b)
 	return quark - GPOINTER_TO_INT (b);
 }
 
+/**
+ * eom_image_get_supported_mime_types:
+ *
+ * Gets the list of supported mimetypes
+ *
+ * Returns: (transfer none)(element-type utf8): a #GList of supported mimetypes
+ **/
 GList *
 eom_image_get_supported_mime_types (void)
 {
@@ -2112,7 +2261,7 @@ eom_image_iter_advance (EomImage *img)
 
 	if ((new_frame = gdk_pixbuf_animation_iter_advance (img->priv->anim_iter, NULL)) == TRUE)
 	  {      
-		g_mutex_lock (priv->status_mutex);
+		g_mutex_lock (&priv->status_mutex);
 		g_object_unref (priv->image);
 		priv->image = gdk_pixbuf_animation_iter_get_pixbuf (priv->anim_iter);
 	 	g_object_ref (priv->image);
@@ -2124,7 +2273,7 @@ eom_image_iter_advance (EomImage *img)
 			priv->width = gdk_pixbuf_get_width (transformed);
 			priv->height = gdk_pixbuf_get_height (transformed);
 		}      
-		g_mutex_unlock (priv->status_mutex);
+		g_mutex_unlock (&priv->status_mutex);
 		/* Emit next frame signal so we can update the display */
 		g_signal_emit (img, signals[SIGNAL_NEXT_FRAME], 0,
 			       gdk_pixbuf_animation_iter_get_delay_time (priv->anim_iter));
@@ -2185,10 +2334,10 @@ eom_image_start_animation (EomImage *img)
 	if (!eom_image_is_animation (img) || priv->is_playing)
 		return FALSE;
 
-	g_mutex_lock (priv->status_mutex);
+	g_mutex_lock (&priv->status_mutex);
 	g_object_ref (priv->anim_iter);
 	priv->is_playing = TRUE;
-	g_mutex_unlock (priv->status_mutex);
+	g_mutex_unlock (&priv->status_mutex);
 
  	g_timeout_add (gdk_pixbuf_animation_iter_get_delay_time (priv->anim_iter), private_timeout, img);
 
@@ -2226,12 +2375,22 @@ eom_image_get_transform (EomImage *img)
  * eom_image_file_changed:
  * @img: a #EomImage
  *
- * Emits EomImage::file-changed signal
+ * Marks the image files contents as changed. Also, emits
+ * EomImage::file-changed signal
  **/
 void
 eom_image_file_changed (EomImage *img)
 {
 	g_return_if_fail (EOM_IS_IMAGE (img));
 
+	img->priv->file_is_changed = TRUE;
 	g_signal_emit (img, signals[SIGNAL_FILE_CHANGED], 0);
 }
+
+gboolean
+eom_image_is_file_changed (EomImage *img)
+{
+	g_return_val_if_fail (EOM_IS_IMAGE (img), TRUE);
+
+	return img->priv->file_is_changed;
+}
