@@ -38,12 +38,23 @@
 #include <stdlib.h>
 #include <string.h>
 
+#if GTK_CHECK_VERSION (3, 4, 3)
+enum {
+  PROP_0,
+  PROP_ORIENTATION
+};
+#endif
+
 #define EOM_THUMB_VIEW_SPACING 0
 
 #define EOM_THUMB_VIEW_GET_PRIVATE(object)				\
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOM_TYPE_THUMB_VIEW, EomThumbViewPrivate))
 
+#if GTK_CHECK_VERSION (3, 4, 3)
+static void eom_thumb_view_init (EomThumbView *thumbview);
+#else
 G_DEFINE_TYPE (EomThumbView, eom_thumb_view, GTK_TYPE_ICON_VIEW);
+#endif
 
 static EomImage* eom_thumb_view_get_image_from_path (EomThumbView      *thumbview,
 						     GtkTreePath       *path);
@@ -51,15 +62,147 @@ static EomImage* eom_thumb_view_get_image_from_path (EomThumbView      *thumbvie
 static void      eom_thumb_view_popup_menu          (EomThumbView      *widget,
 						     GdkEventButton    *event);
 
+#if GTK_CHECK_VERSION (3, 4, 3)
+static void      eom_thumb_view_update_columns (EomThumbView *view);
+
+G_DEFINE_TYPE_WITH_CODE (EomThumbView, eom_thumb_view, GTK_TYPE_ICON_VIEW,
+						 G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL));
+#endif
+
+static gboolean
+thumbview_on_query_tooltip_cb (GtkWidget  *widget,
+			       gint        x,
+			       gint        y,
+			       gboolean    keyboard_mode,
+			       GtkTooltip *tooltip,
+			       gpointer    user_data);
+static void
+thumbview_on_parent_set_cb (GtkWidget *widget,
+#if GTK_CHECK_VERSION (3, 0, 0)
+			    GtkWidget *old_parent,
+#else
+			    GtkObject *oldparent,
+#endif
+			    gpointer   user_data);
+
+static void
+thumbview_on_drag_data_get_cb (GtkWidget        *widget,
+			       GdkDragContext   *drag_context,
+			       GtkSelectionData *data,
+			       guint             info,
+			       guint             time,
+			       gpointer          user_data);
+
 struct _EomThumbViewPrivate {
 	gint start_thumb; /* the first visible thumbnail */
 	gint end_thumb;   /* the last visible thumbnail  */
 	GtkWidget *menu;  /* a contextual menu for thumbnails */
 	GtkCellRenderer *pixbuf_cell;
+	gint visible_range_changed_id;
+
+#if GTK_CHECK_VERSION (3, 4, 3)
+	GtkOrientation orientation;
+	gint n_images;
+	gulong image_add_id;
+	gulong image_removed_id;
+#endif
 };
 
 /* Drag 'n Drop */
 
+static void
+eom_thumb_view_constructed (GObject *object)
+{
+	EomThumbView *thumbview;
+
+	if (G_OBJECT_CLASS (eom_thumb_view_parent_class)->constructed)
+		G_OBJECT_CLASS (eom_thumb_view_parent_class)->constructed (object);
+
+	thumbview = EOM_THUMB_VIEW (object);
+
+	thumbview->priv->pixbuf_cell = gtk_cell_renderer_pixbuf_new ();
+
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (thumbview),
+				    thumbview->priv->pixbuf_cell,
+				    FALSE);
+
+	g_object_set (thumbview->priv->pixbuf_cell,
+#if !GTK_CHECK_VERSION (3, 16, 0)
+	              "follow-state", FALSE,
+#endif
+	              "height", 100,
+	              "width", 115,
+	              "yalign", 0.5,
+	              "xalign", 0.5,
+	              NULL);
+
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (thumbview),
+					thumbview->priv->pixbuf_cell,
+					"pixbuf", EOM_LIST_STORE_THUMBNAIL,
+					NULL);
+
+	gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (thumbview),
+					  GTK_SELECTION_MULTIPLE);
+
+	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (thumbview),
+					  EOM_THUMB_VIEW_SPACING);
+
+	gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (thumbview),
+				       EOM_THUMB_VIEW_SPACING);
+
+	g_object_set (thumbview, "has-tooltip", TRUE, NULL);
+
+	g_signal_connect (thumbview,
+			  "query-tooltip",
+			  G_CALLBACK (thumbview_on_query_tooltip_cb),
+			  NULL);
+
+	thumbview->priv->start_thumb = 0;
+	thumbview->priv->end_thumb = 0;
+	thumbview->priv->menu = NULL;
+
+	g_signal_connect (G_OBJECT (thumbview), "parent-set",
+			  G_CALLBACK (thumbview_on_parent_set_cb), NULL);
+
+	gtk_icon_view_enable_model_drag_source (GTK_ICON_VIEW (thumbview), 0,
+						NULL, 0,
+						GDK_ACTION_COPY);
+	gtk_drag_source_add_uri_targets (GTK_WIDGET (thumbview));
+
+	g_signal_connect (G_OBJECT (thumbview), "drag-data-get",
+			  G_CALLBACK (thumbview_on_drag_data_get_cb), NULL);
+}
+
+static void
+eom_thumb_view_dispose (GObject *object)
+{
+	EomThumbViewPrivate *priv = EOM_THUMB_VIEW (object)->priv;
+#if GTK_CHECK_VERSION (3, 4, 3)
+	GtkTreeModel *model;
+#endif
+
+	if (priv->visible_range_changed_id != 0) {
+		g_source_remove (priv->visible_range_changed_id);
+		priv->visible_range_changed_id = 0;
+	}
+
+#if GTK_CHECK_VERSION (3, 4, 3)
+	model = gtk_icon_view_get_model (GTK_ICON_VIEW (object));
+
+	if (model && priv->image_add_id != 0) {
+		g_signal_handler_disconnect (model, priv->image_add_id);
+		priv->image_add_id = 0;
+	}
+
+	if (model && priv->image_removed_id) {
+		g_signal_handler_disconnect (model, priv->image_removed_id);
+		priv->image_removed_id = 0;
+	}
+#endif
+
+	G_OBJECT_CLASS (eom_thumb_view_parent_class)->dispose (object);
+}
+
 static void
 eom_thumb_view_finalize (GObject *object)
 {
@@ -70,42 +213,92 @@ eom_thumb_view_finalize (GObject *object)
 	G_OBJECT_CLASS (eom_thumb_view_parent_class)->finalize (object);
 }
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-eom_thumb_view_dispose (GObject *object)
-{
-	EomThumbView *thumbview;
-	g_return_if_fail (EOM_IS_THUMB_VIEW (object));
-	thumbview = EOM_THUMB_VIEW (object);
-
-	G_OBJECT_CLASS (eom_thumb_view_parent_class)->dispose (object);
-}
-#else
 static void
+#if GTK_CHECK_VERSION(3, 0, 0)
+eom_thumb_view_destroy (GtkWidget *object)
+#else
 eom_thumb_view_destroy (GtkObject *object)
+#endif
 {
 	EomThumbView *thumbview;
 	g_return_if_fail (EOM_IS_THUMB_VIEW (object));
 	thumbview = EOM_THUMB_VIEW (object);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GTK_WIDGET_CLASS (eom_thumb_view_parent_class)->destroy (object);
+#else
 	GTK_OBJECT_CLASS (eom_thumb_view_parent_class)->destroy (object);
+#endif
+}
+
+#if GTK_CHECK_VERSION (3, 4, 3)
+static void
+eom_thumb_view_get_property (GObject    *object,
+			     guint       prop_id,
+			     GValue     *value,
+			     GParamSpec *pspec)
+{
+	EomThumbView *view = EOM_THUMB_VIEW (object);
+
+	switch (prop_id)
+	{
+	case PROP_ORIENTATION:
+		g_value_set_enum (value, view->priv->orientation);
+		break;
+
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+eom_thumb_view_set_property (GObject      *object,
+			     guint         prop_id,
+			     const GValue *value,
+			     GParamSpec   *pspec)
+{
+	EomThumbView *view = EOM_THUMB_VIEW (object);
+
+	switch (prop_id)
+	{
+	case PROP_ORIENTATION:
+		view->priv->orientation = g_value_get_enum (value);
+#if GTK_CHECK_VERSION (3, 4, 3)
+		eom_thumb_view_update_columns (view);
+#endif
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
 }
 #endif
 
 static void
 eom_thumb_view_class_init (EomThumbViewClass *class)
 {
-	#if GTK_CHECK_VERSION(3, 0, 0)
-	GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-
-	gobject_class->constructed = eom_thumb_view_init;
-	gobject_class->dispose = eom_thumb_view_dispose;
-	#else
 	GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+#if GTK_CHECK_VERSION(3, 0, 0)
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
+#else
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
+#endif
+	gobject_class->constructed = eom_thumb_view_constructed;
+	gobject_class->dispose = eom_thumb_view_dispose;
+#if GTK_CHECK_VERSION (3, 4, 3)
+	gobject_class->get_property = eom_thumb_view_get_property;
+	gobject_class->set_property = eom_thumb_view_set_property;
 
+	g_object_class_override_property (gobject_class, PROP_ORIENTATION,
+	                                  "orientation");
+#endif
 	gobject_class->finalize = eom_thumb_view_finalize;
+#if GTK_CHECK_VERSION(3, 0, 0)
+	widget_class->destroy = eom_thumb_view_destroy;
+#else
 	object_class->destroy = eom_thumb_view_destroy;
-	#endif
+#endif
 
 	g_type_class_add_private (class, sizeof (EomThumbViewPrivate));
 }
@@ -182,14 +375,15 @@ eom_thumb_view_update_visible_range (EomThumbView *thumbview,
 	priv->end_thumb = end_thumb;
 }
 
-static void
-thumbview_on_visible_range_changed_cb (EomThumbView *thumbview,
-				       gpointer user_data)
+static gboolean
+visible_range_changed_cb (EomThumbView *thumbview)
 {
 	GtkTreePath *path1, *path2;
 
+	thumbview->priv->visible_range_changed_id = 0;
+
 	if (!gtk_icon_view_get_visible_range (GTK_ICON_VIEW (thumbview), &path1, &path2)) {
-		return;
+		return FALSE;
 	}
 
 	if (path1 == NULL) {
@@ -205,49 +399,40 @@ thumbview_on_visible_range_changed_cb (EomThumbView *thumbview,
 
 	gtk_tree_path_free (path1);
 	gtk_tree_path_free (path2);
+
+	return FALSE;
+}
+
+static void
+eom_thumb_view_visible_range_changed (EomThumbView *thumbview)
+{
+	if (thumbview->priv->visible_range_changed_id == 0) {
+		g_idle_add ((GSourceFunc)visible_range_changed_cb, thumbview);
+	}
+
+}
+
+static void
+thumbview_on_visible_range_changed_cb (EomThumbView *thumbview,
+				       gpointer user_data)
+{
+	eom_thumb_view_visible_range_changed (thumbview);
 }
 
 static void
 thumbview_on_adjustment_changed_cb (EomThumbView *thumbview,
 				    gpointer user_data)
 {
-	GtkTreePath *path1, *path2;
-	gint start_thumb, end_thumb;
-
-	if (!gtk_icon_view_get_visible_range (GTK_ICON_VIEW (thumbview), &path1, &path2)) {
-		return;
-	}
-
-	if (path1 == NULL) {
-		path1 = gtk_tree_path_new_first ();
-	}
-	if (path2 == NULL) {
-		gint n_items = gtk_tree_model_iter_n_children (gtk_icon_view_get_model (GTK_ICON_VIEW (thumbview)), NULL);
-		path2 = gtk_tree_path_new_from_indices (n_items - 1 , -1);
-	}
-
-	start_thumb = gtk_tree_path_get_indices (path1) [0];
-	end_thumb = gtk_tree_path_get_indices (path2) [0];
-
-	eom_thumb_view_add_range (thumbview, start_thumb, end_thumb);
-
-	/* case we added an image, we need to make sure that the shifted thumbnail is cleared */
-	eom_thumb_view_clear_range (thumbview, end_thumb + 1, end_thumb + 1);
-
-	thumbview->priv->start_thumb = start_thumb;
-	thumbview->priv->end_thumb = end_thumb;
-
-	gtk_tree_path_free (path1);
-	gtk_tree_path_free (path2);
+	eom_thumb_view_visible_range_changed (thumbview);
 }
 
 static void
 thumbview_on_parent_set_cb (GtkWidget *widget,
-				#if GTK_CHECK_VERSION(3, 0, 0)
+#if GTK_CHECK_VERSION(3, 0, 0)
 			    GtkWidget *old_parent,
-				#else
+#else
 			    GtkObject *old_parent,
-			    #endif
+#endif
 			    gpointer   user_data)
 {
 	EomThumbView *thumbview = EOM_THUMB_VIEW (widget);
@@ -367,11 +552,7 @@ thumbview_get_tooltip_string (EomImage *image)
 	ExifData *exif_data;
 #endif
 
-#if GLIB_CHECK_VERSION (2, 30, 0)
 	bytes = g_format_size (eom_image_get_bytes (image));
-#else
-	bytes = g_format_size_for_display (eom_image_get_bytes (image));
-#endif
 
 	eom_image_get_size (image, &width, &height);
 
@@ -381,6 +562,7 @@ thumbview_get_tooltip_string (EomImage *image)
 				       0, NULL, NULL);
 	g_object_unref (file);
 	if (file_info == NULL) {
+		g_free (bytes);
 		return NULL;
 	}
 
@@ -428,7 +610,7 @@ thumbview_get_tooltip_string (EomImage *image)
 		gchar time_buffer[32];
 
 		date = eom_exif_util_format_date (
-			eom_exif_util_get_value (exif_data, EXIF_TAG_DATE_TIME_ORIGINAL, time_buffer, 32));
+			eom_exif_data_get_value (exif_data, EXIF_TAG_DATE_TIME_ORIGINAL, time_buffer, 32));
 
 		if (date) {
 			extra_info = g_strdup_printf ("\n%s %s", _("Taken on"), date);
@@ -526,55 +708,11 @@ eom_thumb_view_init (EomThumbView *thumbview)
 {
 	thumbview->priv = EOM_THUMB_VIEW_GET_PRIVATE (thumbview);
 
-	thumbview->priv->pixbuf_cell = gtk_cell_renderer_pixbuf_new ();
-
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (thumbview),
-	      	  		    thumbview->priv->pixbuf_cell,
-	      			    FALSE);
-
-	g_object_set (thumbview->priv->pixbuf_cell,
-	              "follow-state", FALSE,
-	              "height", 100,
-	              "width", 115,
-	              "yalign", 0.5,
-	              "xalign", 0.5,
-	              NULL);
-
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (thumbview),
-	                                thumbview->priv->pixbuf_cell,
-	      		                "pixbuf", EOM_LIST_STORE_THUMBNAIL,
-	                                NULL);
-
-	gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (thumbview),
- 					  GTK_SELECTION_MULTIPLE);
-
-	gtk_icon_view_set_column_spacing (GTK_ICON_VIEW (thumbview),
-					  EOM_THUMB_VIEW_SPACING);
-
-	gtk_icon_view_set_row_spacing (GTK_ICON_VIEW (thumbview),
-				       EOM_THUMB_VIEW_SPACING);
-
-	g_object_set (thumbview, "has-tooltip", TRUE, NULL);
-
-	g_signal_connect (thumbview,
-			  "query-tooltip",
-			  G_CALLBACK (thumbview_on_query_tooltip_cb),
-			  NULL);
-
-	thumbview->priv->start_thumb = 0;
-	thumbview->priv->end_thumb = 0;
-	thumbview->priv->menu = NULL;
-
-	g_signal_connect (G_OBJECT (thumbview), "parent-set",
-			  G_CALLBACK (thumbview_on_parent_set_cb), NULL);
-
-	gtk_icon_view_enable_model_drag_source (GTK_ICON_VIEW (thumbview), 0,
-						NULL, 0,
-						GDK_ACTION_COPY);
-	gtk_drag_source_add_uri_targets (GTK_WIDGET (thumbview));
-
-	g_signal_connect (G_OBJECT (thumbview), "drag-data-get",
-			  G_CALLBACK (thumbview_on_drag_data_get_cb), NULL);
+	thumbview->priv->visible_range_changed_id = 0;
+#if GTK_CHECK_VERSION (3, 4, 3)
+	thumbview->priv->image_add_id = 0;
+	thumbview->priv->image_removed_id = 0;
+#endif
 }
 
 /**
@@ -594,6 +732,45 @@ eom_thumb_view_new (void)
 	return GTK_WIDGET (thumbview);
 }
 
+#if GTK_CHECK_VERSION (3, 4, 3)
+static void
+eom_thumb_view_update_columns (EomThumbView *view)
+{
+	EomThumbViewPrivate *priv;
+
+	g_return_if_fail (EOM_IS_THUMB_VIEW (view));
+
+	priv = view->priv;
+
+	if (priv->orientation == GTK_ORIENTATION_HORIZONTAL)
+			gtk_icon_view_set_columns (GTK_ICON_VIEW (view),
+			                           priv->n_images);
+}
+
+static void
+eom_thumb_view_row_inserted_cb (GtkTreeModel    *tree_model,
+                                GtkTreePath     *path,
+                                GtkTreeIter     *iter,
+                                EomThumbView    *view)
+{
+	EomThumbViewPrivate *priv = view->priv;
+
+	priv->n_images++;
+	eom_thumb_view_update_columns (view);
+}
+
+static void
+eom_thumb_view_row_deleted_cb (GtkTreeModel    *tree_model,
+                               GtkTreePath     *path,
+                               EomThumbView    *view)
+{
+	EomThumbViewPrivate *priv = view->priv;
+
+	priv->n_images--;
+	eom_thumb_view_update_columns (view);
+}
+#endif
+
 /**
  * eom_thumb_view_set_model:
  * @thumbview: A #EomThumbView.
@@ -607,13 +784,52 @@ void
 eom_thumb_view_set_model (EomThumbView *thumbview, EomListStore *store)
 {
 	gint index;
+#if GTK_CHECK_VERSION (3, 4, 3)
+	EomThumbViewPrivate *priv;
+	GtkTreeModel *existing;
+#endif
 
 	g_return_if_fail (EOM_IS_THUMB_VIEW (thumbview));
 	g_return_if_fail (EOM_IS_LIST_STORE (store));
 
+#if GTK_CHECK_VERSION (3, 4, 3)
+	priv = thumbview->priv;
+
+	existing = gtk_icon_view_get_model (GTK_ICON_VIEW (thumbview));
+
+	if (existing != NULL) {
+		if (priv->image_add_id != 0) {
+			g_signal_handler_disconnect (existing,
+			                             priv->image_add_id);
+		}
+		if (priv->image_removed_id != 0) {
+			g_signal_handler_disconnect (existing,
+			                             priv->image_removed_id);
+
+		}
+	}
+
+	priv->image_add_id = g_signal_connect (G_OBJECT (store), "row-inserted",
+	                            G_CALLBACK (eom_thumb_view_row_inserted_cb),
+	                            thumbview);
+	priv->image_removed_id = g_signal_connect (G_OBJECT (store),
+	                             "row-deleted",
+	                             G_CALLBACK (eom_thumb_view_row_deleted_cb),
+	                             thumbview);
+
+	thumbview->priv->n_images = eom_list_store_length (store);
+#endif
+
 	index = eom_list_store_get_initial_pos (store);
 
+#if GTK_CHECK_VERSION (3, 4, 3)
+	gtk_icon_view_set_model (GTK_ICON_VIEW (thumbview),
+	                         GTK_TREE_MODEL (store));
+
+	eom_thumb_view_update_columns (thumbview);
+#else
 	gtk_icon_view_set_model (GTK_ICON_VIEW (thumbview), GTK_TREE_MODEL (store));
+#endif
 
 	if (index >= 0) {
 		GtkTreePath *path = gtk_tree_path_new_from_indices (index, -1);
@@ -677,7 +893,7 @@ eom_thumb_view_get_n_selected (EomThumbView *thumbview)
  * Gets the #EomImage stored in @thumbview's #EomListStore at the position indicated
  * by @path.
  *
- * Returns: A #EomImage.
+ * Returns: (transfer full): A #EomImage.
  **/
 static EomImage *
 eom_thumb_view_get_image_from_path (EomThumbView *thumbview, GtkTreePath *path)
@@ -737,7 +953,7 @@ eom_thumb_view_get_first_selected_image (EomThumbView *thumbview)
  * Gets a list with the currently selected images. Note that a new reference is
  * hold for each image and the list must be freed with g_list_free().
  *
- * Returns: A newly allocated list of #EomImage's.
+ * Returns: (element-type EomImage) (transfer full): A newly allocated list of #EomImage's.
  **/
 GList *
 eom_thumb_view_get_selected_images (EomThumbView *thumbview)
@@ -845,7 +1061,7 @@ eom_thumb_view_select_single (EomThumbView *thumbview,
 			path = gtk_tree_path_new_from_indices (n_items - 1, -1);
 			break;
 		case EOM_THUMB_VIEW_SELECT_RANDOM:
-			path = gtk_tree_path_new_from_indices ((int)(((float)(n_items - 1) * rand()) / (float)(RAND_MAX + 1.f)), -1);
+			path = gtk_tree_path_new_from_indices (g_random_int_range (0, n_items), -1);
 			break;
 		}
 	} else {
@@ -883,7 +1099,7 @@ eom_thumb_view_select_single (EomThumbView *thumbview,
 			break;
 		case EOM_THUMB_VIEW_SELECT_RANDOM:
 			gtk_tree_path_free (path);
-			path = gtk_tree_path_new_from_indices ((int)(((float)(n_items - 1) * rand()) / (float)(RAND_MAX + 1.f)), -1);
+			path = gtk_tree_path_new_from_indices (g_random_int_range (0, n_items), -1);
 			break;
 		}
 	}
