@@ -197,7 +197,7 @@ eom_uri_converter_set_property (GObject      *object,
 		break;
 
 	case PROP_SPACE_CHARACTER:
-		priv->space_character = g_value_get_char (value);
+		priv->space_character = g_value_get_schar (value);
 		break;
 
 	case PROP_COUNTER_START:
@@ -249,7 +249,7 @@ eom_uri_converter_get_property (GObject    *object,
 		break;
 
 	case PROP_SPACE_CHARACTER:
-		g_value_set_char (value, priv->space_character);
+		g_value_set_schar (value, priv->space_character);
 		break;
 
 	case PROP_COUNTER_START:
@@ -834,8 +834,6 @@ eom_uri_converter_preview (const char *format_str, EomImage *img, GdkPixbufForma
 
 	n_digits = ceil (MIN (log10 (G_MAXULONG), MAX (log10 (counter), log10 (n_images))));
 
-	str = g_string_new ("");
-
 	if (!g_utf8_validate (format_str, -1, NULL))
 	    return NULL;
 
@@ -843,6 +841,8 @@ eom_uri_converter_preview (const char *format_str, EomImage *img, GdkPixbufForma
 	s = format_str;
 	token_next = FALSE;
 
+	str = g_string_new ("");
+
 	for (i = 0; i < len; i++, s = g_utf8_next_char (s)) {
 		c = g_utf8_get_char (s);
 
